/* #define DEBUGTRACE_ENABLED */
#include "debugtrace.hpp"
#include "lasp_config.h"
#if LASP_HAS_ULDAQ == 1
#include "lasp_uldaq.h"
#include "lasp_uldaq_impl.h"
#include <uldaq.h>

void fillUlDaqDeviceInfo(std::vector<DeviceInfo> &devinfolist,
                         void *vDescriptors) {

  DEBUGTRACE_ENTER;

  DaqDeviceDescriptor *descriptors_copy =
      static_cast<DaqDeviceDescriptor *>(vDescriptors);

  UlError err;
  unsigned int numdevs = MAX_ULDAQ_DEV_COUNT_PER_API;

  DaqDeviceDescriptor devdescriptors[MAX_ULDAQ_DEV_COUNT_PER_API];
  DaqDeviceDescriptor descriptor;
  DaqDeviceInterface interfaceType = ANY_IFC;

  err = ulGetDaqDeviceInventory(interfaceType, devdescriptors,
                                static_cast<unsigned *>(&numdevs));

  if (err != ERR_NO_ERROR) {
    throw rte("UlDaq device inventarization failed");
  }

  for (unsigned i = 0; i < numdevs; i++) {

    descriptor = devdescriptors[i];

    // Copy structure over, if given as not nullptr
    if (descriptors_copy) {
      descriptors_copy[i] = descriptor;
    }

    DeviceInfo devinfo;
    devinfo.api = uldaqapi;
    string name, interface;
    if (string(descriptor.productName) != "DT9837A") {
      throw rte("Unknown UlDAQ type");
    }

    switch (descriptor.devInterface) {
    case USB_IFC:
      name = "USB - ";
      break;
    case BLUETOOTH_IFC:
      /* devinfo. */
      name = "Bluetooth - ";
      break;

    case ETHERNET_IFC:
      /* devinfo. */
      name = "Ethernet - ";
      break;
    default:
      name = "Uknown interface = ";
    }

    name += string(descriptor.productName) + " " + string(descriptor.uniqueId);
    devinfo.device_name = std::move(name);

    devinfo.physicalOutputQty = DaqChannel::Qty::Voltage;

    devinfo.api_specific_devindex = i;
    devinfo.availableDataTypes.push_back(
        DataTypeDescriptor::DataType::dtype_fl64);
    devinfo.prefDataTypeIndex = 0;

    devinfo.availableSampleRates = {8000,  10000, 11025, 16000, 20000,
                                    22050, 24000, 32000, 44056, 44100,
                                    47250, 48000, 50000, 50400, 51000};

    devinfo.prefSampleRateIndex = 11;

    devinfo.availableFramesPerBlock = {512, 1024, 2048, 4096, 8192};

    devinfo.availableInputRanges = {1.0, 10.0};
    devinfo.prefInputRangeIndex = 0;

    devinfo.ninchannels = 4;
    devinfo.noutchannels = 1;

    devinfo.hasInputIEPE = true;
    devinfo.hasInputACCouplingSwitch = true;
    devinfo.hasInputTrigger = true;

    devinfo.hasInternalOutputMonitor = true;

    // Finally, this devinfo is pushed back in list
    devinfolist.push_back(devinfo);
  }
}

std::unique_ptr<Daq> createUlDaqDevice(const DeviceInfo &devinfo,
                                       const DaqConfiguration &config) {
  return std::make_unique<DT9837A>(devinfo, config);
}

#endif // LASP_HAS_ULDAQ
