// lasp_threadedaps.h
//
// Author: J.A. de Jong - ASCEE
//
// Description: Real Time Spectrum Viewer
#pragma once
#include "lasp_avpowerspectra.h"
#include "lasp_filter.h"
#include "lasp_mathtypes.h"
#include "lasp_threadedindatahandler.h"
#include <memory>
#include <mutex>

/**
 * \addtogroup dsp
 * @{
 *
 * \addtogroup rt
 * @{
 */

class RtAps : public ThreadedInDataHandler {

  std::unique_ptr<Filter> _filterPrototype;
  std::vector<std::unique_ptr<Filter>> _freqWeightingFilters;

  /**
   * @brief Storage for sensitivity values
   */
  vd _sens;

  /**
   * @brief Mutex only for _ps. Other members are only accessed in thread.
   */
  mutable std::mutex _ps_mtx;
  AvPowerSpectra _ps;

public:
  /**
   * @brief Initialize RtAps.
   *
   * @param mgr StreamMgr singleton reference
   * @param freqWeightingFilter Optionally: the frequency weighting filter.
   * Nullptr should be given for Z-weighting.
   * @param For all other arguments, see constructor of AvPowerSpectra
   */
  RtAps(StreamMgr &mgr, const Filter *freqWeightingFilter, const us nfft = 2048,
        const Window::WindowType w = Window::WindowType::Hann,
        const d overlap_percentage = 50., const d time_constant = -1);
  ~RtAps();

  /**
   * @brief Get the latest estimate of the power spectra
   *
   * @return If available, the latest estimate. If no estimate available, an
   * empty ccube().
   */
  ccube getCurrentValue() const;

  bool inCallback_threaded(const DaqData &) override final;
  void reset(const Daq *) override final;
};

/** @} */
/** @} */
